@@ -1,7 +1,7 @@
 import logging
 import re
 from threading import local
-from typing import Iterable, Optional, NamedTuple, Union
+from typing import Iterable, Optional, NamedTuple
 
 _LOGGER = logging.getLogger(__name__)
 _RE_BASIC_ACK = re.compile('^basic.ack$', re.IGNORECASE)
@@ -30,12 +30,15 @@ class Message(NamedTuple):
     :param routing_key: RabbitMQ routing key
     :param body: The message's body
     :param properties: Message properties (see :class:`pika.BasicProperties`)
+    :param mandatory: If `True`, requires the message to be added to
+      at least one queue.
     """
 
     exchange: str
     routing_key: str
-    body: Union[str, bytes]
-    properties: MessageProperties
+    body: bytes
+    properties: Optional[MessageProperties] = None
+    mandatory: bool = False
 
 
 class DeliverySet:
@@ -106,6 +109,7 @@ class Publisher:
 
         headers = {'header1': 'value1', 'header2': 'value2'}
         properties = rabbitmq.MessageProperties(
+            delivery_mode=2,  # This makes the message persistent!
             app_id='example-publisher',
             content_type='application/json',
             headers=headers,
@@ -113,7 +117,8 @@ class Publisher:
         m1 = rabbitmq.Message(exchange='', routing_key='test',
                               body='Message 1', properties=properties)
         m2 = rabbitmq.Message(exchange='', routing_key='test',
-                              body='Message 2', properties=properties)
+                              body='Message 2', properties=properties,
+                              mandatory=True)
 
         mq = rabbitmq.Publisher(app)
         mq.publish_messages([m1, m2])
@@ -225,6 +230,7 @@ class Publisher:
         _LOGGER.info('Channel opened')
         self._channel = channel
         channel.add_on_close_callback(self._on_channel_closed)
+        channel.add_on_return_callback(self._on_return_callback)
 
         # Send the Confirm.Select RPC method to RabbitMQ to enable
         # delivery confirmations on the channel. The only way to turn
@@ -251,6 +257,14 @@ class Publisher:
         if self._channel is closed_channel:
             self._kill_connection()
 
+    def _on_return_callback(self, channel, method, properties, body):
+        """This method is invoked by pika when basic_publish is sent a message
+        that has been rejected and returned by the server. For
+        example, this happens when a message with a `mandatory=True`
+        flag can not be routed to any queue.
+        """
+        self._state.returned_messages = True
+
     def _on_delivery_confirmation(self, method_frame):
         """This method is invoked by pika when RabbitMQ responds to a
         Basic.Publish RPC command, passing in either a Basic.Ack or
@@ -288,6 +302,8 @@ class Publisher:
         if pending.all_confirmed:
             if state.received_nack:
                 state.error = DeliveryError('received nack')
+            elif state.returned_messages:
+                state.error = DeliveryError('returned messages')
             else:
                 state.error = None
             connection.ioloop.stop()
@@ -304,9 +320,10 @@ class Publisher:
         n = len(messages)
         state.pending_deliveries = DeliverySet(state.message_number + 1, n)
         state.received_nack = False
+        state.returned_messages = False
         state.message_number += n
         for m in messages:
-            channel.basic_publish(m.exchange, m.routing_key, m.body, m.properties)
+            channel.basic_publish(m.exchange, m.routing_key, m.body, m.properties, m.mandatory)
         _LOGGER.debug('Published %i messages', len(messages))
 
     def publish_messages(
